
* ---------------------------------------------------------------- smcxtend
       Program smcxtend

* Extends a smc file by either adding zeros to front and back of by reflecting
* the time series around the beginning and ending points (as advocated by
* Hudson, 1979; I am allowing this to text his method, which I think is a 
* mistaken way of extending the time series becauseif filtered and then
* stripped of the extended sections, the filter transients are not accounted
* for properly and integration will most likely lead to drifts.  I am allowing
* this option here to check the accuracy of my assumption.  For that
* application, use this program to extend the time series, use blpadflt to 
* filter (but without padding), smc_snip to snip out the original section,
* and then smc2vd to integrate).

* For flexibility, information is read from a control file.
* Here is an example:
*!Control file for program SMCXTend
*!
*!Extend with zeros (Y/N)?  If "N", then use reflection (Hudson).
* Y
*! Detrend, using straight line fit to first and last points, before extending?
* N
*! Replace discarded portions of original data with zeros (Y/N)?  By doing this,
*! two horizontal components with the same number of time points before
*! padding (the usual case) will have the same number after padding and can
*! be rotated.
* Y
*!Length of leading pad in seconds (BAP suggests 1.5*nroll/fc):
* 9.0
*!Go to first zero crossing?
* Y
*!Use first point if smaller than first zero crossing?
* Y
*!Length of trailing pad in seconds (BAP suggests 1.5*nroll/fc):
* 9.0
*!Go to last zero crossing?
* Y
*!Use last point if smaller than last zero crossing?
* Y
*! Remove mean before determining segment to be padded?
*! If Yes, the mean will be determined from the whole record.  If this
*! is not desireable, remove the mean outside of this program
* N
*! Remove mean after determining segment to be padded (the segment
*! will probably be different than the segment originally used to determine the
*! mean, and therefore the mean will no longer be 0.0. But then after removing
*! this mean, the segment to be padded will not be at 0.0 values, but I hope this
*! is a small problem.  I probably should iterate):
* Y
*!character string to append to filename:
* .zxtend
*!Name of SMC file (output name = stem.pad):
* 319A.SMC
* 319B.SMC
* 319C.SMC
* 334A.SMC
* 334B.SMC
* 334C.SMC
*stop


*  Dates:  06/21/04 - Written by D. Boore (based on SMC_Pad)
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8



      real  rhead(50)

      integer ihead(48)

      real x(:), y(:)
      allocatable :: x, y

      character chead(11)*80, comments(200)*80
      character f_ctl*40, f_out*70, f_smc*40
      character buf_stop*4, buf*10
      character ctl_cmmnts(10)*80
      character string_to_append*40

      character date_c*8, time_c*11

      logical f_ctl_exist, zcross_l, zcross_t, use_first, use_last
      logical remove_mean_before, remove_mean_after,
     :  replace_discarded_data_with_zeros, extend_with_zeros
      logical detrend

* Open Input file:

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smcxtend.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smcxtend.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        extend_with_zeros = .true.
      else
        extend_with_zeros  = .false.
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        detrend = .true.
      else
        detrend = .false.
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        replace_discarded_data_with_zeros = .true.
      else
        replace_discarded_data_with_zeros = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, n_ctl_cmmnts)
      read(nu_ctl,*) tlz

      call skipcmnt(nu_ctl, ctl_cmmnts, n_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        zcross_l = .true.
      else
        zcross_l = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, n_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        use_first = .true.
      else
        use_first = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, n_ctl_cmmnts)
      read(nu_ctl,*) ttz

      call skipcmnt(nu_ctl, ctl_cmmnts, n_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        zcross_t = .true.
      else
        zcross_t = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, n_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        use_last = .true.
      else
        use_last = .false.
      end if

      write(*,'(a)') ' tlz, ttz = '
      write(*, *)      tlz, ttz

      write(*,'(a)') ' zcross_l, zcross_t = '
      write(*, *)      zcross_l, zcross_t

      write(*,'(a)') ' use_first, use_last = '
      write(*, *)      use_first, use_last


      call skipcmnt(nu_ctl, ctl_cmmnts, n_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        remove_mean_before = .true.
      else
        remove_mean_before = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_before = ', remove_mean_before

      call skipcmnt(nu_ctl, ctl_cmmnts, n_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) .eq. 'Y') then
        remove_mean_after = .true.
      else
        remove_mean_after = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_after = ', remove_mean_after

* GET STRING TO APPEND TO FILE NAME:

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      string_to_append = ' '
      read(nu_ctl,'(a)') string_to_append
      call trim_c(string_to_append, nc_s2appnd)

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

1000  continue

      f_smc = ' '
      read(nu_ctl,'(a)') f_smc
      call trim_c(f_smc,nc_f_smc)
      buf_stop = ' '
      buf_stop = f_smc(1:4)
      call upstr(buf_stop)

      if(buf_stop .eq. 'STOP') then
        close(nu_ctl)
        stop
      end if

      write(*,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)

      call smc_nsps(f_smc(1:nc_f_smc), npts_smc, sps)

      if (extend_with_zeros) then

        nlz = tlz * sps
        npts_plus_lz = npts_smc + nlz

        ntz = ttz * sps
        npts_plus_lz_tz = npts_smc + nlz + ntz

        npts4alloc = npts_plus_lz_tz
        allocate(x(npts4alloc), y(npts_smc))

        tskip = 0.0
        tlength = -1
        call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :                   y, chead, ihead, rhead, comments)

      precision = ' '
      if (ihead(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
        npts_in = npts_smc

        ncomments = ihead(16)
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :     '| To input file '//f_smc(1:nc_f_smc)//
     :     ', program SMCXTend did the following:'

        if (detrend) then
          ystrt = y(1)
          slope = (y(npts_in)-y(1))/(npts_in-1)
          do i = 1, npts_in
            y(i) = y(i) - (ystrt+slope*float(i-1))
          end do
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :     '| removed a line fit to first and last points and'
        end if


        do i = 1, npts4alloc
          x(i) = 0.0
        end do

        do i = 1, npts_in
          x(i) = y(i)
        end do



        call smcpadf(x, npts_in, sps, nlz, ntz, 
     :  replace_discarded_data_with_zeros,
     :  npts_out, izcl, izct,  
     :  remove_mean_before, remove_mean_after, 
     :  zcross_l, use_first, zcross_t, use_last,
     :  f_smc, nc_f_smc, comments, ncomments)    

      else

        npts4alloc = 3* npts_smc
        allocate(x(npts4alloc), y(npts_smc))

        do i = 1, npts4alloc
          x(i) = 0.0
        end do

        tskip = 0.0
        tlength = -1
        call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :                   y, chead, ihead, rhead, comments)

      precision = ' '
      if (ihead(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
        npts_in = npts_smc

        if (remove_mean_before) then        
          n_start = 1
          n_stop = npts_in
          icorr_cumavg_before = 0
          call Get_Avg(y, n_start, n_stop, npts_in, 
     :                   sps, icorr_cumavg_before, avg_before)
          do i = n_start, n_stop
            y(i) = y(i) - avg_before
          end do
          n_start_before = n_start
          n_stop_before = n_stop
        end if

        ncomments = ihead(16)
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :     '| To input file '//f_smc(1:nc_f_smc)//
     :     ', program SMCXTend did the following:' 
        
        if (remove_mean_before) then        
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1p,e11.4)') 
     :   '| determined and removed the mean', avg_before
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i5,1x,a,1x,i5,a)') 
     :   '| between indices ', n_start_before, ' and ',
     :        n_stop_before, ' before extending the time series'
        end if

        if (detrend) then
          ystrt = y(1)
          slope = (y(npts_in)-y(1))/(npts_in-1)
          do i = 1, npts_in
            y(i) = y(i) - (ystrt+slope*float(i-1))
          end do
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :     '| * removed a line fit to first and last points;'
        end if


        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :     '| * extended the original record '//
     :         'by reflecting around the first and last points'

        do i = 1, npts_smc
          x(i+npts_smc-1) = y(i)
          x(npts_smc-i+1) = y(i)
          x(npts_smc+npts_smc-2+i) = y(npts_smc+1-i)
        end do

        npts_out = npts_smc + npts_smc - 2 + npts_smc

      end if


* Store new ncomments, npts:

      ihead(16) = ncomments            
      ihead(17) = npts_out

* Construct file name for output
      f_out = ' '
      f_out = f_smc(1:nc_f_smc)//string_to_append(1:nc_s2appnd)
      call trim_c(f_out, nc_f_out)
      write(*,'(a)') ' f_out = '//f_out(1:nc_f_out)

      ncomments = ncomments + 1
      write(comments(ncomments),'(a)') 
     :     '| resulting in output file '//f_out(1:nc_f_out)
      iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
      call datetime(date_c, time_c, iflag)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :     '| on '//date_c//' at time '//time_c

      ncomments = ncomments + 1
      comments(ncomments) = '|'

* Write the output file:

      itype = -1
      call SMCWrite(f_out, precision,
     :               x, chead, itype, 
     :                    ihead, rhead, comments)

* That should be it; deallocate the y array and loop back for another file
 
      deallocate(x, y)

      goto 1000

      end
* ---------------------------------------------------------------- smcxtend


      include '\forprogs\smcpadf.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\get_avg.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmax.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\smc_nsps.for'
      include '\forprogs\datetime.for'
      include '\forprogs\skipcmnt.for'

                     