      Program Env2Hist

* Reads a series of files containing envelopes (made by 
* \forprogs\env2phs) and computes an average histogram 

* Dates: 06/21/02 - Written by D. Boore, based on PhD2Hist

      real env(:)
      allocatable :: env

      integer icount(200)
      real bin_center(200)
      real sum_icount(200), avg1(200), avg2(200), avg3(200)
      real hfreq(200), cumulative(200)      

      character buf*80, buf_upper*4, 
     :        f_hist*70, f_env*70, cmnts2skip(30)*79

      character f_ctl*40
      logical f_ctl_exist, f_env_exist, spec_bin

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=env2Hist.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'env2Hist.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl,status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      read(nu_ctl,*) bin_start, bin_stop, dbin

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_hist = ' '
      read(nu_ctl,'(a)') f_hist
      call trim_c(f_hist,nc_f_hist)
      call get_lun(nu_hist)
      open(unit=nu_hist,file=f_hist(1:nc_f_hist),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      do i = 1, 200
        avg1(i) = 0.0
        avg2(i) = 0.0
        avg3(i) = 0.0
        sum_icount(i) = 0.0
      end do

      num_files = 0

1000  continue
      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') go to 1000
      if(buf_upper(1:4) .eq. 'STOP') go to 8888  ! finish computing averages

      f_env = ' '
      f_env = buf(1:nc_buf)
      nc_f_env = nc_buf     
      inquire(file=f_env(1:nc_f_env), exist=f_env_exist)
      if (.not. f_env_exist) then
          write(*,'(a)') ' ****** FILE '//f_env(1:nc_f_env)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          go to 1000
      end if

* Passed tests; proceed

      num_files = num_files + 1

      call get_lun(nu_env)
      open(nu_env, file = f_env(1:nc_f_env), status='unknown')      

      call skipcmnt(nu_env,cmnts2skip,nc_cmnts2skip)

      read(nu_env,*) idum1, nout

      call skip(nu_env,2)

      allocate( env(nout) )

* Now read the data from the file:

      write(*,'(1x,a)') 
     :   'Processing file:'//f_env(1:nc_f_env)

      do i = 1, nout
        read(nu_env,*) time, ts, env(i)
      end do
      
      close(nu_env)

      write(*,'(a,1x,1p, e10.3)') ' To check: env(1) = ', env(1)

      spec_bin = .true.
      call histfreq(env, nout, dbin, spec_bin, bin_start, bin_stop,
     :              bin_center, icount, hfreq, cumulative, nbins, ncum)


* Accumulate averages:
*  1) for each bin, with no normalization
*  2) for each bin, but normalization each histogram so area = 1
*  3) for each bin, but average hfreq

      do i = 1, nbins
        avg1(i) = avg1(i) + float(icount(i))
        avg2(i) = avg2(i) + float(icount(i))/cumulative(nbins)
        avg3(i) = avg3(i) + hfreq(i)
      end do 

      deallocate( env)
      close(nu_env)

      go to 1000

8888  continue

* Finish computation of final averages and print to nu_hist
      do i = 1, nbins
        sum_icount(i) = avg1(i)
        avg1(i) = avg1(i)/float(num_files)
        avg2(i) = avg2(i)/float(num_files)
        avg3(i) = avg3(i)/float(num_files)
      end do 

* Normalize each to unit area:
      cum1 = 0.0
      cum2 = 0.0
      cum3 = 0.0
      do i = 1, nbins
        cum1 = cum1 + avg1(i)
        cum2 = cum2 + avg2(i)
        cum3 = cum3 + avg3(i)
      end do

      do i = 1, nbins
        avg1(i) = avg1(i)/(dbin*cum1)
        avg2(i) = avg2(i)/(dbin*cum2)
        avg3(i) = avg3(i)/(dbin*cum3)
      end do

      write(nu_hist,'(4x,a,2x,a,2x,a,3x,a,3x,a,3x,a)') 
     :   'bin_left', 'bin_center', 
     :   'sum_icount', 'countnorm', 'cntnrmnrm', 'hfreqnorm'      

      do i = 1, nbins
        write(nu_hist,'(1p, 6(1x,e11.4))')  
     :  bin_center(i) - dbin/2, bin_center(i), 
     :  sum_icount(i), avg1(i), avg2(i), avg3(i)
      end do      

9999  continue

      close(nu_hist)
      close(nu_ctl)

      stop
      end

      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\histfreq.for'

       