* ---------------------------------------------------------------- smc_snip
      Program SMC_Detrend

* Detrend the time series by subtracting a line fit to the first and last points.
* Might want to generalize this later.

* Input/output controlled by information in a control file.  Here is an example:

*!Control file for program SMC_Detrend
*! Revision of program involving a change in the control file on this date:
*   12/20/10
*!Summary file name:
* smc_detrend.sum
*!
*!PROCESSING PARAMETERS AND FILES, IN GROUPS
*!
*pp: new set of parameters
*!tskip tlength  
* 0.0 400.0
*!mean: type (-1=specify,0=none,1=standard), t4mean_strt, t4mean_stop, 
*!mean (use if type = -1):
* 1  0.0 8.0  -999.
*!Baseline option:
*!  1 = a line fit to the first and last points is removed; note that the mean 
*!      need not be removed before detrending if type = 0 above.  In addition, 
*!      caution should be used in using this option for "normal" time series; it
*!      is intended as a quick baseline correction for the infrequent cases in which
*!      there is a large linear trend in the time series
*!  2 = Fit a polynomial
* 2
*!Order of polynomial(1:line;2:quad.;3:cubic, etc-- 11 max) :
*  3  
*!Array showing coefficients fixed at 0 (0) or free (1); must be norder+1 
*!values:
* 0 0 1 1   ! cubic, with first and second terms constrained to zero
*!T1b, T1e, T2b, T2e (fit between T1b--T1e and T2b--T2e):
*  0.0 0.0 56.0 200.0
*!Torigin: if <= 0.0, the polynomial has the equation
*!            y = c0 + c1*t + c2*t^2 + ... + cn*t^n
*!         if > 0.0, the polynomial has the equation
*!            y = 0.0, t < torigin
*!            y = c0 + c1*(t-torigin) + ... + cn*(t-torigin)^n
*!               In this case, c0 should probably be fixed as 0.0
* 0.0
*!Percent taper to apply to front (usually 0.0)
* 0.0
*!character string to append to filename to make the output filename:
*! if this character string equals "<auto>", the extension will be constructed
*! from the input parameters).
* <auto>
*!f_smc_in ("STOP" in any column to quit)
* file1_in.smc  
* file2_in.smc 
* file3_in.smc 
*StoP
   
* Dates: 02/13/09 - Written by D. Boore
!        12/20/10 - Broadened the types of detrending to include plynomials,
!                   using blpadflt as a guide.
 
      external dfpoly

      parameter (npc = 12)   ! max order of polynomial plus 1
      parameter (small = 0.01) 

      character char_head(11)*80, comments(500)*80, 
     :                            ctl_cmmnts(100)*79,
     :                            date_ctl_correct*8, date_ctl_in*30
     
      integer int_head(48), option
      
      integer nc, ic_in(npc)
      real*8 c(npc),covar(npc,npc)  ! allows up to npc-1 order polynomial fit
      real*8 pd(3)            ! allows up to 2 derivatives of the poly fit
      real*8 chisq, t_dp
      real real_head(50)
      
      integer status

      real y_in(:), y_bl(:), y_out(:), time_r(:)
      allocatable :: y_in, y_bl, y_out, time_r

      real*8 x(:), y(:), sig(:)
      allocatable :: x, y, sig

      double precision avg
      character f_ctl*60, f_sum*60, f_in*220, buf*300, buf_c*79
      character f_stem*200, f_out*220
      character string_to_append*100, buf3*3, buf4*4, buf6*6, f_pad*120
      character compute_v_d*10, precision*8

      logical f_ctl_exist, f_in_exist, rmv_trnd, add_impulse
      logical remove_mean_before, remove_mean_after, detrend
      
      character header(27)*12, damp_c*2
      character date_c*10, time_c*11

      pi = 4.0*atan(1.)
      twopi = 2.0*pi

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smc_detrend.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smc_detrend.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '12/20/10'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
      write(nu_sum,'(a)') ' Summary file for program smc_detrend'
      write(nu_sum,'(a)') 
     :     '   Using control file: '//f_ctl(1:nc_f_ctl)

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

! CHECK FOR ANOTHER FILE TO PROCESS

1000  continue

        write(nu_sum,*)  
        
        buf = ' '
        read(nu_ctl,'(a)', iostat=status) buf
        if (status /= 0) then
          go to 9999
        end if
        write(nu_sum,'(1x,a)') buf
        call trim_c(buf,nc_buf)
        buf3 = ' '
        buf3 = buf(1:3)
        call upstr(buf3)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if (buf4 == 'STOP') go to 9999
        if (buf3 == 'PP:') then
! New processing parameters

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          read(nu_ctl,*) tskip, tlength      
          write(nu_sum,'(1x,a,2(1x,es10.3))') 
     :         ' tskip, tlength=', tskip, tlength
      
! PARAMETERS FOR OBTAINING THE MEAN:

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          read(nu_ctl,*) itype_mean, 
     :                 t4mean_strt_in, t4mean_stop_in, spec_mean
          write(nu_sum,*)  itype_mean, 
     :                   t4mean_strt_in, t4mean_stop_in, spec_mean

! PARAMETERS FOR BASELINE ADJUSTMENT:

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          read(nu_ctl,*) option
          write(nu_sum,*) option

          if (option /= 1 .and. option /=2) then
            write(*,*) ' Invalid baseline option; QUITTING'
            go to 9999
          end if
          
          if (option == 1) then
            detrend = .true.
          else
            detrend = .false.
          end if
           
          
          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do

          read(nu_ctl,*)  iorder 
          write(nu_sum,*) iorder 

          if (.not.detrend) then        
            if (iorder == 0) then
              write(nu_sum,'(a)') ' ERROR!!! iorder = 0; '//
     :         'Skip to next set of processing parameters!!!)'
              write(*,'(a)') ' ERROR!!! iorder = 0; '//
     :         'Skip to next set of processing parameters!!!)'
              go to 1000
            end if   
          end if

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) (ic_in(i),i=1,iorder+1)

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) t1b, t1e, t2b, t2e
          write(nu_sum,*) t1b, t1e, t2b, t2e

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) torigin 
          write(nu_sum,*) torigin 

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do

          read(nu_ctl,*)  prcntfrtpr
          write(nu_sum,*) prcntfrtpr
        
! GET STRING TO APPEND TO FILE NAME:

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          string_to_append = ' '
          read(nu_ctl,'(a)') string_to_append
          call trim_c(string_to_append, nc_s2appnd)
          buf6 = ' '
          buf6 = string_to_append(1:nc_s2appnd)
          call upstr(buf6)
          
          if (buf6(1:6) .eq. '<AUTO>') then ! construct extension
          
            if (detrend) then
              string_to_append = ' '
              string_to_append = '.bl_line_fit_to_end_points.smc'
            else
              string_to_append = ' '
              string_to_append = '.bl_poly_'
              write(string_to_append(10:10),'(i1)') iorder
              string_to_append = string_to_append(1:10)//'.smc'
            end if
            
            call trim_c(string_to_append, nc_s2appnd) 
          
           end if
          

          write(nu_sum,'(a)') ' string_to_append: '//
     :          string_to_append(1:nc_s2appnd)

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

          go to 1000

        end if  ! end loop over "PP:"

! Read the input file:

        f_in = ' '
        call trim_c(buf, nc_buf)
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)

        f_in_exist = .false.
        
        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ** Input file '//f_in(1:nc_f_in)//
     :    ' does not exist, moving to next file **'
          write(nu_sum,'(a)') ' ** Input file '//f_in(1:nc_f_in)//
     :    ' does not exist, moving to next file **'
          go to 1000
        end if

        write(*,'(a)') ' Processing file: '//f_in(1:nc_f_in)
        write(nu_sum,*)  
        write(nu_sum,'(a)') ' Processing file: '//f_in(1:nc_f_in)

        call smc_npts(f_in(1:nc_f_in), npts_max, sps)
        allocate (y_in(npts_max), y_bl(npts_max), 
     :            x(npts_max), y(npts_max), sig(npts_max), 
     :            time_r(npts_max)) 

        call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :               y_in, char_head, int_head, real_head, comments)
        sps = real_head(2)
        dt = 1.0/sps
        ncomments = int_head(16)
        npts = int_head(17)
        tend = float(npts-1)/sps

        write(nu_sum,'(a)') ' sps, ncomments, npts = '
        write(nu_sum, *) sps, ncomments, npts

! Write comment headers (NOTE: I am distributing these throughout
! rather than in one place because smcpadf writes
! comment within the subroutine, and I wanted the comments in proper order.)

        call get_date(date_c)
        call get_time(time_c)

        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '| Use program SMC_Detrend to process input file '
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '| '//f_in(1:nc_f_in)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '|   on '//date_c//' at time '//time_c
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,es10.3)') 
     :  '|    tskip = ', tskip
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,es10.3)') 
     :  '|    tlength = ', tlength
     
 
        npts4alloc = npts_max + 2
        
 
        write(nu_sum,'(a)')  
     :     ' npts, npts_max, npts4alloc ='
        write(nu_sum,*) 
     :       npts, npts_max, npts4alloc 

        allocate (y_out(npts4alloc))
 
        do i = 1, npts4alloc
          y_out(i) = 0.0
        end do

! Step 1: REMOVE MEAN FROM WHOLE RECORD

!  Determine the appropriate mean:

        t4mean_strt = t4mean_strt_in
        t4mean_stop = t4mean_stop_in
        
        if (itype_mean .eq. 0) then
          n_t4mean = 0
          avg_all = 0.0
          ncomments = ncomments + 1
          comments(ncomments) = 
     :    '|   no mean removed.'
        else if (itype_mean .eq. -1) then   ! use specified mean
          avg_all = spec_mean
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :    '|   remove the specified mean ', avg_all,
     :      ' from whole record.'
        else if (itype_mean .eq. 1) then
          if (t4mean_strt .lt. 0.0) t4mean_strt = 0.0
          if (t4mean_stop .gt. tend) t4mean_stop = tend
          n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
          n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
          n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
          t4mean = float(n_t4mean -1)/sps
          avg = 0.0d0
          do i = n_t4mean_strt, n_t4mean_stop
            avg = avg + dble(y_in(i))
          end do
          avg_all = sngl(avg/dble(n_t4mean))
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :    '|    remove the computed mean = ', avg_all,
     :        ' from whole record;'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1p,e12.5,a,e12.5)') 
     :    '|    mean determined from average between t= ', 
     :     t4mean_strt, ' and t = ', t4mean_stop
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i7)') 
     :    '|    corresponding to n_t4mean_strt = ', n_t4mean_strt
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i7)') 
     :    '|                     n_t4mean_stop = ', n_t4mean_stop
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i7)') 
     :    '|                      and n_t4mean = ', n_t4mean
        else            ! shouldn't be here!
          write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :         improper value of itype_mean.  itype_mean = ', 
     :         itype_mean, ' QUITTING!!'
          write(*) '  Should not be here, '//'
     :         improper value of itype_mean.  itype_mean = ', 
     :         itype_mean, ' QUITTING!!'
          go to 9999
        end if

!  Remove the mean from the whole record:
        write(nu_sum,'(a,1x,i6)') ' n_t4mean = ', n_t4mean
        write(nu_sum,'(a,1x,1pe12.5,1x,a)') 
     :   ' Removing mean = ', avg_all, ' from the whole record'

        do i = 1, npts
          y_out(i) = y_in(i) - avg_all
          time_r(i) = float(i-1)*dt
        end do

* Step baseline determination and removal:  

        SELECT CASE(detrend)
        
        CASE(.true.)
        
          call rmvtrend(y_out, npts)
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f8.3)') 
     :        '|   Remove baseline' 
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i2)') 
     :        '|   Baseline is a a line fit to the first & last points'
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          int_head(16) = ncomments
          
        CASE(.false.)
        
*   First extract the portion to fit:

! first work with beginning section (if there is one):
        n_seg1 = 0
        if (t1e <= 0.0) then
          n_t1b = 0
          n_t1e = 0
          ntaper1 = 0
        else
          n_t1b = nint(sps * t1b) + 1
          if (t1e > tend) t1e = tend
          n_t1e = nint(sps * t1e) + 1
          t1b = float(n_t1b-1)/sps
          t1e = float(n_t1e-1)/sps
          n_seg1 = n_t1e - n_t1b + 1 
          ntaper1 = float(n_seg1)  * prcntfrtpr/100.0
            do i = 1, n_seg1
              x(i) = time_r(i + n_t1b - 1) - torigin    ! time origin = torigin
              y(i) = wind_box(i,1,n_seg1,ntaper1) * y_out(i + n_t1b - 1)
              sig(i) = 1.0
            end do
        end if

! next work with second section (if there is one):
        n_seg2 = 0
        if (t2e <= 0.0) then
          n_t2b = 0
          n_t2e = 0
          ntaper2 = 0
        else
          if (t2b >= tend) then
            write(*,'(a,f6.1,a,f6.1,a)')
     :       ' ** t2b = ', t2b, ', >= tend = ', tend,
     :       ', moving to next file **'
            go to 1000
          end if
          n_t2b = nint(sps * t2b) + 1
          if (t2e > tend) t2e = tend
          n_t2e = nint(sps * t2e) + 1
          t2b = float(n_t2b-1)/sps
          t2e = float(n_t2e-1)/sps
          n_seg2 = n_t2e - n_t2b + 1 
          ntaper2 = float(n_seg2)  * prcntfrtpr/100.0
            do i = 1, n_seg2
              x(i+n_seg1) = time_r(i + n_t2b - 1) - torigin !time orgn = torigin
              y(i+n_seg1) = wind_box(i,1,n_seg2,ntaper2) * 
     :                      y_out(i + n_t2b - 1)
              sig(i+n_seg1) = 1.0
            end do
        end if
          
        n_tfit = n_seg2 + n_seg1

*   Now fit the curve:
     
        nc = iabs(iorder) + 1

        do i = 1, npc
          c(i)     = 0.0
        end do                

        call dlfit(x, y, sig, n_tfit, c, ic_in, nc,
     :     covar, npc, chisq, dfpoly)

        deallocate (x, y, sig)
        
          write(nu_sum,'(a, i2, a)') 
     :     ' Fit order ', iorder, ' polynomial to input time series: '
          write(nu_sum,'(a)') 
     :     '   torigin, prcntfrtpr, '//
     :     't1b, t1e, n_seg1, ntaper1, '//
     :     't2b, t2e, n_seg2, ntaper2, '//
     :     'n_tfit = '
         write(nu_sum,'(2(1x,f8.3), 
     :       2(1x,f8.3), 2(1x,i5),
     :       2(1x,f8.3), 2(1x,i5),
     :       1x,i5)') 
     :       torigin, prcntfrtpr, 
     :       t1b, t1e, n_seg1, ntaper1,
     :       t2b, t2e, n_seg2, ntaper2,
     :       n_tfit
       
        write(nu_sum,'(t17,a)') 
     :        ' and: coefficients = '
        write(nu_sum,'(1p12(1x,e12.5))') (c(i),i=1, nc)

 
*   Determine and subtract acceleration baseline:

* Initialize the baseline:
        do i = 1, npts
          y_bl(i) = 0.0
        end do

        if (torigin <= 0.0) then 
          n_start_bl = 1
        else
          n_start_bl = int(torigin/dt) + 1
        end if
        
        write(nu_sum, '(a)') ' torigin, dt, n_start_bl = '
        write(nu_sum, *)       torigin, dt, n_start_bl
        
* Fill out a_bl from n_start_bl to end:
        do i = n_start_bl, npts
          t_dp = time_r(i)-torigin
          nd = 1 ! no derivatives desired, but set to 1 to avoid runtime error
                 ! pd(1) = polynomial evaluated at t_dp
          call dddpoly(c, nc, t_dp, pd, nd)
          y_bl(i) = pd(1)
        end do

        do i = 1, npts
          y_out(i) = y_out(i) - y_bl(i)
        end do
        npts_out = npts
        
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f8.3)') 
     :        '|   Remove baseline, starting at torigin =', torigin
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i2)') 
     :        '|   Baseline is a polynomial of order ', iabs(iorder)
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,12(1x,i1))') 
     :        '|   with array ic_in (0=coefficient fixed at 0.0) = ', 
     :        (ic_in(i), i = 1, nc)
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f8.3,a,f8.3,a)') 
     :        '|   fit between ', t1b, ' and ', t1e, ' sec and'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f8.3,a,f8.3,a)') 
     :        '|   fit between ', t2b, ' and ', t2e, ' sec'//
     :        ' with regression coefficients: '
          ncomments = ncomments + 1
          nc_up = min0(6, nc)
          write(comments(ncomments),'(a,1p,6(1x,e12.5))') 
     :        '| ', (c(i),i=1, nc_up)
          if (nc .gt. 6) then
            ncomments = ncomments + 1
            write(comments(ncomments),'(a,1p,6(1x,e12.5))') 
     :        '| ', (c(i),i=nc_up+1, nc)
          end if

          ncomments = ncomments + 1
          comments(ncomments) = '|'
          int_head(16) = ncomments
          
        END SELECT
 
      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      f_out = ' '
      f_out = f_in(1:nc_f_in)//
     :            string_to_append(1:nc_s2appnd)
      call trim_c(f_out, nc_f_out)
      
      itype = 0
      call SMCWrite(f_out(1:nc_f_out), precision,
     :     y_out, char_head, itype, int_head, real_head, comments)

      deallocate (y_in, y_bl, y_out)

      goto 1000  ! loop over input time series

9999  continue

      close(nu_ctl)
      close(nu_sum)


      end
* ---------------------------------------------------------------- smc_snip
      
      include '\forprogs\get_lun.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\SMCRead.for'
      include '\forprogs\SMCWrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\smc_npts.for'

      include '\forprogs\rmvtrend.for'     

      include '\forprogs\dlfit.for'
      include '\forprogs\dgaussj.for'
      include '\forprogs\dcovsrt.for'
      include '\forprogs\dddpoly.for'
      include '\forprogs\dfpoly.for'

      include '\forprogs\wind_box.for'      

      include '\forprogs\get_date.for'
      include '\forprogs\get_time.for'
      
!      include '\forprogs\ddpoly.for'
!      include '\forprogs\gaussj.for' 
!      include '\forprogs\covsrt.for'
!      include '\forprogs\fpoly.for'
!      include '\forprogs\lfit.for' 

!      include '\forprogs\getf_out.for'
!      include '\forprogs\detrend.for' 

