! ------------------------------------------Begin SMC_Rot_1Pair_Per_Azm
      Program SMC_Rot_1Pair_per_Azm

! Read 2 smc files, an azimuth, and output file names and rotate

! Structure of control file:
! Line 1 follows:
!! Control file for program SMC_Rot_1Pair_per_Azm
!! Revision of program involving a change in the control file on this date:
!   02/08/18
!! Summary file name:
!   smc_rot_1pair_per_azm.sum
!pp: ----- processing parameters for a new pair of files follow this line -------
!! Names of smc input files (one file per line):
!   0516a.smc
!   0516c.smc
!! Azimuth of the first output file (the other is this plus 90):
!   32.0
!! Name of smc output files:
!   0516r1.smc
!   0516r2.smc
!pp: ----- processing parameters for a new pair of files follow this line -------
!! Names of smc input files (one file per line):
!   0526a.smc
!   0526c.smc
!! Azimuth of the first output file (the other is this plus 90):
!  278.0
!! Name of smc output files:
!   0526r1.smc
!   0526r2.smc
! Stop   (starting in any column)

! Dates: 02/20/00 - Written by D. Boore
!        11/10/00 - Increase max_dimen from 30000 to 100000)
!        12/09/00 - Change control file, use dynamically allocatable
!                   arrays
!        07/17/03 - Add sps to smc_npts
!        10/07/06 - Modernize treatment of character strings for lf95
!        10/07/06 - Changed "tskip" input parameter in smcread from
!                   "1" to "0.0".
!        10/17/07 - Transparent change to user needed because smcread can
!                   read two input formats
!        03/24/08 - Changed name from "smcrtold.for"
!        11/20/09 - Changed name of default control file.
!                 - Trap unspecified input orientation and set to 0, 90.
!        01/27/18 - Some modernization of syntax
!        02/07/18 - Use subroutine rotate_separate_in_out_variables rather 
!                   than subroutine rotate
!        02/08/18 - Added date to control file

      character precision*8

      character date_ctl_correct*8, date_ctl_in*30
     

      character char_head(11,2)*80, comments(200,2)*80, buf4*4
      integer int_head(48,2), nc_f_in(2)
      
      integer npts(2)
      real real_head(50,2)

      real a_in(:,:), a_out(:,:)
      allocatable :: a_in, a_out

      real rnull(2), azm_in(2), azm_out(2)
      character buf_c*80, f_ctl*80, f_sum*80
      character  f_in(2)*80, f_out*80, c_az*7
      logical f_ctl_exist

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file '//
     :      '(cr = SMC_Rot_1Pair_Per_Azm.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl == ' ') f_ctl = 'SMC_Rot_1Pair_Per_Azm.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do


      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skip(nu_ctl,2)

! Check date of control file:
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '02/08/18'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        write(*,'(a)') 
     :     ' The date for the control file you used is '
     :       //date_ctl_in(1:nc_date_ctl_in)
        write(*,'(a)') 
     :     ' The correct date is '
     :       //date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
! Check date of control file:

      call skip(nu_ctl,1)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      print *,' f_sum = '//f_sum(1:nc_f_sum)
      
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') 
     :             ' Summary file for program SMC_Rot_1Pair_Per_Azm'
      write(nu_sum,'(2a)') '   Using control file: ', f_ctl(1:nc_f_ctl)

      loop_over_files: DO

        buf_c = ' '
        read(nu_ctl,'(a)',IOSTAT=iostatus) buf_c
        if (iostatus /= 0) EXIT
        write(nu_sum,'(1x,a)') buf_c
        call trim_c(buf_c,nc_buf_c)
        buf4 = ' '
        buf4 = buf_c(1:4)
        call upstr(buf4)

        if (buf4 == 'STOP') EXIT

        buf_c = ' '
        read(nu_ctl,'(a)') buf_c
        call trim_c(buf_c,nc_buf_c)
        write(nu_sum,'(1x,a)') buf_c(1:nc_buf_c)

! First get max dimension and allocate arrays:
        do j = 1, 2
          f_in(j) = ' '
          read(nu_ctl,'(a)') f_in(j)
          call trim_c(f_in(j), nc_f_in(j))
          
          print *,' f_in = '//f_in(j)(1:nc_f_in(j))
          
          write(nu_sum,'(1x,a)') f_in(j)(1:nc_f_in(j))

          call smc_npts(f_in(j)(1:nc_f_in(j)), npts(j), sps)
        end do

        max_dimen = max0(npts(1), npts(2))
        allocate( a_in(max_dimen,2), a_out(max_dimen,2))

! Second, loop over input files, reading data:
        do j = 1, 2
          call SMCRead(f_in(j)(1:nc_f_in(j)), 0.0, 100000.0,
     :                 a_in(1,j), char_head(1,j), 
     :                 int_head(1,j), real_head(1,j), 
     :                 comments(1,j))
          precision = ' '
          if (int_head(47,j) == 8) then
            precision = 'HIGH'
          else
            precision = 'STANDARD'
          end if
      
          npts(j)  = int_head(17,j)  ! should check equality of both time series
          rnull(j) = real_head(1,j)
          inull = int_head(1,j)
          if (int_head(14,j) == inull) then
            azm_in(j) = float(j-1)*90.0
          else  
            azm_in(j) = real(int_head(14,j))
          end if
          print *,' Input az = ',azm_in(j)

        end do

! Check for equality of npts:
        if ( npts(1) /= npts(2)) then
          write(nu_sum,'(a,2(1x,i6))') 
     :      ' QUITTING: npts1 /= npts2; npts1, npts2 = ', 
     :        npts(1), npts(2)
          close(unit=nu_sum)
          close(nu_ctl)
          stop
        end if

! Check for equality of rnull:
        if ( rnull(1) /= rnull(2)) then
          write(nu_sum,'(a,1p2(1x,e12.5))') 
     :      ' QUITTING: rnull1 /= rnull2; rnull1, rnull2 = ', 
     :        rnull(1), rnull(2)
          close(unit=nu_sum)
          close(nu_ctl)
          stop
        end if

        buf_c = ' '
        read(nu_ctl,'(a)') buf_c
        call trim_c(buf_c,nc_buf_c)
        write(nu_sum,'(1x,a)') buf_c(1:nc_buf_c)

        read(nu_ctl,*) azmr
        print *,' azmr = ', azmr
        
        call Rotate_Separate_In_Out_Variables( a_in(1,1), a_in(1,2),
     :               rnull(2),
     :               azm_in(1), azm_in(2), azmr, npts(2), 
     :               a_out(1,1), a_out(1,2), azm_out(1), azm_out(2))

        buf_c = ' '
        read(nu_ctl,'(a)') buf_c
        call trim_c(buf_c,nc_buf_c)
        write(nu_sum,'(1x,a)') buf_c(1:nc_buf_c)

        do j = 1,2
          f_out = ' '
          read(nu_ctl,'(a)') f_out
          call trim_c(f_out,nc_f_out)
          write(nu_sum,'(1x,a)') f_out(1:nc_f_out)
          int_head(14,j) = nint(azm_out(j))
          c_az = ' '
          write(c_az,'(f7.1)') azm_out(j)
          char_head(6,j)(53:80) = ' '
          write(char_head(6,j)(53:57),'(i5)') int_head(14,j)
          ncomments = int_head(16,j)
          comments(ncomments+1,j) = '|'
          ncomments = ncomments + 1
          comments(ncomments+1,j) = '| files '//f_in(1)(1:nc_f_in(1))//
     :                               ' and '//f_in(2)(1:nc_f_in(2))
          ncomments = ncomments + 1
          comments(ncomments+1,j) = '| rotated to azimuth '//c_az//
     :                     ' using program SMC_Rot_1Pair_Per_Azm'
          ncomments = ncomments + 1
          comments(ncomments+1,j) = '|'
          ncomments = ncomments + 1
          int_head(16,j) = ncomments

          print *,' int_head(14)(16)(17) = ', 
     :       int_head(14,j),int_head(16,j),int_head(17,j)


          call SMCWrite(f_out(1:nc_f_out), precision,
     :               a_out(1,j),
     :              char_head(1,j), -1, 
     :              int_head(1,j), real_head(1,j), comments(1,j))

        end do
        deallocate( a_in, a_out)

      END DO loop_over_files   ! loop back for another pair of components

      close(unit=nu_sum)
      close(nu_ctl)
      stop

      end
! ------------------------------------------End SMC_Rot_1Pair_Per_Azm

      include '\forprogs\rotate_separate_in_out_variables.for'     
      include '\forprogs\get_lun.for'     
      include '\forprogs\skip.for'     
      include '\forprogs\smcread.for'     
      include '\forprogs\smcwrite.for'     
      include '\forprogs\mnmaxidx.for'     
      include '\forprogs\trim_c.for'     
      include '\forprogs\upstr.for'     
      include '\forprogs\smc_npts.for'     
      include '\forprogs\distaz.for'   


