
* ---------------------------------------------------------------- SMCNuHdr
       Program SMCNuHdr

*  Changes header information of preexisting SMC files

* This version will only change integer or real headers

*  For flexibility, information is read from a control file.
*  Here is an example:
*!Control file for program SMCNuHdr
*! As many comment lines as desired, each starting with "!"
*! a list of smc files:
*! i_format (0 or 1), i_int, i_real:
*!   If i_format = 0 the filename in the list below will contain 
*!                 i_int integer headers and i_real real headers.
*!   If i_format = 1, the list of headers to be changed, one header per line,
*!         is followed by a list of files (but placeholders have to be
*!         included for i_int and i_real (e.g., 1 0 0).
*! Note: i(13) = vorient; i(14) = horient 
*! Note: i(40) = ieqid; i(41) = istaid 
*! Note: r(11) = stalat; r(12) = stalong 
*! Note: r(33) = low-cut filter frequency 
*! Note: t(01) = 1 UNCORRECTED  (for example)
* 1 0 0   ! need  0 0 because read in free format (used when i_format = 0)
*!
*! headers to be changed, one header per line, with
*! the type and number of header indicated by the first few characters,
*! followed by the list of smc file whose headers will be changed
*!
* i(40)= 345
* i(41)= 1347
* r(06)= 7.3
* testdat1.smc
* testdat2.smc
* testdat3.smc
* i(41) = 3461
* testdat4.smc
*Stop  ! This tells the program where to stop, obviously
*
* 0 1 2  ! 1 = one integer header, 2 = two real headers
* 41     ! index of int_head
* 33     ! index of real_head 
* 34     ! index of real_head 
* testdat1.smc  1347 0.02 -2
* testdat4.smc  3461 0.02 -2
*Stop

*  Dates:  02/26/01 - Written by D. Boore (based on SMCFxHdr and BlFlt2)
*          05/24/01 - Increase dimension of f_smc to 2000. Also, changed
*                     new_set = 0 to new_set = 1 deep in the code
*          05/25/01 - Correct small bug in debig output statement
*          06/09/01 - Reorder input so can avoid the need for an array
*                     of f_smc files.  Also, this is now consistent
*                     with blflt2 and smc2asc.  Also, use subroutine skipcmnt
*                     to skip comments, and combine rs2nuhdr and smcnuhdr.
*          06/13/01 - rc_subs no longer look for "=", so I had to look
*                     explicitly for "=" and parse the buf string beyond this
*                     for the value of the header
*          03/24/02 - Suggest default name for control file.
*          05/30/03 - Allow changes of text headers
*          07/17/03 - Add sps to smc_npts parameter list
*          12/08/03 - Small modificationsso can compile and use with LF95
*          10/17/07 - Transparent change to user needed because smcread can
*                     read two input formats
*          06/14/10 - Minor code clean up (e.g., "/=" rather than ".ne." in if 
*                     statements).
!          11/12/20 - Increase length of filenames (from 40 to 200)

      character precision*8


      real  real_head(50), rnull, anew_r_head(400)

      integer int_head(48), ncomments, inull, 
     :        indx_ihead(400), indx_rhead(400), new_i_head(400), 
     :        indx_thead(11), nc_thead_new(11)

      real rd(100,5), rv(100,5), psv(100,5), aa(100,5)
      real tprd(100,5), tprv(100,5), tpaa(100,5)
      real damp(5), per(110)

      real y(:)
      allocatable :: y

      character char_head(11)*80, comments(400)*80, cmnts_ctl(400)*80
      character f_ctl*200, f_smc*200, f_sum*200
      character buf*80, buf_upper*80, indx_c*2, thead_new(11)*80

      character date_c*8, time_c*11, ifrmt_c*6

      logical f_ctl_exist, last_read_f_smc

      inull = -32768
      rnull = +1.7e+38

* Open Input file:

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smcnuhdr.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smcnuhdr.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

* DEBUG
* Open debug.out
      call get_lun(nu_dbg)
      open(unit=nu_dbg, file='debug.out', status='unknown')
* DEBUG


* Read through until find the line with the input format parameter:
      call skipcmnt(nu_ctl,cmnts_ctl,ncmnts_ctl)

* Read line specifying input format:
      read(nu_ctl,*) i_format, i_int, i_real

* DEBUG
      write(nu_dbg,'(a,3(1x,i3))') 
     :    ' i_format, i_int, i_real = ',
     :      i_format, i_int, i_real
* DEBUG

      if(i_format /= 0 .and. i_format /= 1) then
         write(*,'(a,i3,a)') ' i_format = ', i_format, 
     :   ' which is not 0 or 1, so quitting!' 
         goto 9999
      end if


      if (i_format == 0) then   ! params on same line as filename
*############################################ i_format = 0 ############

* DEBUG
*        write(nu_dbg,'(a)') ' Inside loop for i_format = 0'
* DEBUG
 
        do i = 1, i_int
          read(nu_ctl,*) indx_ihead(i)
        end do

        do i = 1, i_real
          read(nu_ctl,*) indx_rhead(i)
        end do

2000    continue

        buf = ' '
        read(nu_ctl,'(a)',end = 9999) buf
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf
        call upstr(buf_upper)
        if(nc_buf == 0 .or. buf(1:1) == '!') go to 2000
        if(buf_upper(1:4) == 'STOP') then
          close(nu_ctl)
          stop
        end if

        f_smc = ' '
        call rcc(1,buf,nc_buf,f_smc,nc_f_smc,ibgn,iend)
        do i = 1, i_int
          call rci(i+1,buf,nc_buf,new_i_head(i),ibgn,iend,istat)
        end do
        do i = 1, i_real
          call rcf(i+1+i_int,buf,nc_buf,anew_r_head(i),ibgn,iend,istat)
        end do

        write(*,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)

* Get headers, determine if the file is a time series or response spectra file:
        call ReadHdrs(f_smc(1:nc_f_smc), char_head, int_head,
     :                real_head, comments)

        if (char_head(1)(1:1) == '5') then  ! response spectra
          call rs2read(f_smc(1:nc_f_smc), 
     :        damp, ndamp, per, nper, rd, rv, psv, aa, 
     :        char_head, int_head, real_head, comments,
     :        itflag, tprd, tprv, tpaa)

        else
          call SMC_Npts(f_smc(1:nc_f_smc),npts,sps)
          allocate( y(npts))
          tskip = 0.0
          tlength = -1.0
* DEBUG
*        write(nu_dbg,'(a,i1,a,1x,i6)') 
*     :      ' For i_format = ', i_format, 
*     :      ', allocated y with npts = ', npts
* DEBUG

          call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :             y, char_head, int_head, real_head, comments)

      precision = ' '
      if (int_head(47) == 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
        end if

* Modify the headers:
        do i = 1, i_int
          int_head(indx_ihead(i)) = new_i_head(i)
* DEBUG
*          write(nu_dbg,'(a,i2,a,i2,a,i4)') ' for i = ',i,
*     :      ' int_head(', indx_ihead(i),') = ', new_i_head(i)
* DEBUG
        end do
        do i = 1, i_real
          real_head(indx_rhead(i)) = anew_r_head(i)
* DEBUG
*          write(nu_dbg,'(a,i2,a,i2,a,1pe10.3)') ' for i = ',i,
*     :      ' real_head(', indx_rhead(i),') = ', anew_r_head(i)
* DEBUG
        end do

        ncomments = int_head(16)
        iflag = 0 ! not used by current PC version of datetime; Chris
                  ! Stephens would modify datetime.for to include Unix routines
        call datetime(date_c, time_c, iflag)
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        comments(ncomments) = '| Using program SMCNuHDR:'
        do i = 1, i_int
          write(indx_c,'(i2)') indx_ihead(i)          
          ncomments = ncomments + 1
          comments(ncomments) = 
     :  '|   int_head('//indx_c//') replaced on '//date_c//
     :  ' at time '//time_c
        end do
        do i = 1, i_real
          write(indx_c,'(i2)') indx_rhead(i)          
          ncomments = ncomments + 1
          comments(ncomments) = 
     :  '|   real_head('//indx_c//') replaced on '//date_c//
     :  ' at time '//time_c
        end do
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        int_head(16) = ncomments
        itype = -1

        if (char_head(1)(1:1) == '5') then  ! response spectra
          call rs2write(f_smc(1:nc_f_smc), 
     :        damp, ndamp, per, nper, rd, rv, aa, 
     :        char_head, int_head, real_head, comments, 
     :        itflag, tprd, tprv, tpaa)
        else
          call SMCWrite(f_smc(1:nc_f_smc), precision,
     :                y, char_head, itype, 
     :                 int_head, real_head, comments)
          deallocate( y)        
        end if

        go to 2000

      end if
*############################################ i_format = 1 ############
* DEBUG
        write(nu_dbg,'(a)') ' Inside loop for i_format = 1'
* DEBUG
 
* All analysis below assumes that i_format = 1

      
* Read through until find the line with the header info:
      call skipcmnt(nu_ctl,cmnts_ctl,ncmnts_ctl)

      last_read_f_smc = .true.

3000  continue

      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
* DEBUG
*      write(nu_dbg,*)
*      write(nu_dbg,'(2a)') ' buf = ', buf(1:nc_buf)
* DEBUG
      buf_upper = ' '
      buf_upper = buf
      call upstr(buf_upper)
      if(nc_buf == 0 .or. buf(1:1) == '!') go to 3000
      if(buf_upper(1:4) == 'STOP') then
        close(nu_ctl)
        stop
      end if


      indx_equals = index(buf,'=')

      if(indx_equals /= 0) then  ! must be a header; get the header info
         if(last_read_f_smc) then  ! a new set of headers
           last_read_f_smc = .false.
* Reset counters:
           n_ihead = 0
           n_rhead = 0
           n_thead = 0
         end if
* A header value (store all values in arrays)
        call upstr(buf)
        ileft  = index(buf,'(')
        iright = index(buf,')')
        ichars = iright - ileft - 1

* NEED TO FIND WHICH HEADER--- LOOK BETWEEN ( AND )
        if(ichars .eq. 1) ifrmt_c = '(i1)'        
        if(ichars .eq. 2) ifrmt_c = '(i2)'        
        if(ichars .eq. 3) ifrmt_c = '(i3)'        
        if(ichars .eq. 4) ifrmt_c = '(i4)'        
        if(ichars .eq. 5) ifrmt_c = '(i5)'       
        if(ichars .eq. 6) ifrmt_c = '(i6)'   ! out of luck if ichars > 6!

* DEBUG
*      write(nu_dbg,'(a,i3)') ' ichars = ', ichars
* DEBUG

        if(buf(ileft-1:ileft-1) == 'I') then  ! an integer header     
          n_ihead = n_ihead + 1
          read(buf(ileft+1:iright-1),ifrmt_c) indx_ihead(n_ihead)
          call rci(1,buf(indx_equals+1:nc_buf),nc_buf-indx_equals,
     :             new_i_head(n_ihead),ibgn,iend,istat)
* rci looks for first entry beyond '='          
* DEBUG
*      write(nu_dbg,'(a,2(1x,i4))') 
*     :   ' Found int hdr; n_ihead, new_i_head = ', 
*     :     n_ihead, new_i_head(n_ihead)
* DEBUG

        else if(buf(ileft-1:ileft-1) == 'R') then
          n_rhead = n_rhead + 1
          read(buf(ileft+1:iright-1),ifrmt_c) indx_rhead(n_rhead)
          call rcf(1,buf(indx_equals+1:nc_buf),nc_buf-indx_equals,
     :             anew_r_head(n_rhead),ibgn,iend,istat)
* rcf looks for first entry beyond '='          
* DEBUG
*      write(nu_dbg,'(a,1x,i4,1x,1pe10.3))') 
*     :   ' Found real hdr; n_rhead, anew_r_head = ', 
*     :     n_rhead, anew_r_head(n_rhead)
* DEBUG
        else if(buf(ileft-1:ileft-1) == 'T') then
          n_thead = n_thead + 1
          if (n_thead .gt. 11) then
            write(*, '(a)') ' ERROR:  n_thead .gt. 11; QUITTING'
            stop
          end if
          read(buf(ileft+1:iright-1),ifrmt_c) indx_thead(n_thead)
          thead_new(n_thead) = ' '
          thead_new(n_thead) = buf(indx_equals+1:nc_buf)
          call trim_c(thead_new(n_thead), nc_thead_new(n_thead))
        else
             ! ERROR  SKIP GOTO NEXT LINE
          goto 3000
        end if
        
        go to 3000   ! go back for a new line

      else    ! not a new header; must be a smc file, so process it

        last_read_f_smc = .true.

* Get the file name:
        call rcc(1,buf,nc_buf,f_smc,nc_f_smc,ibgn,iend)
* DEBUG
*        write(nu_dbg,*)
*        write(nu_dbg,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)
* DEBUG

        write(*,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)

* Now have list of smc files and the new headers; open each file and 
* replace the headers
        
* DEBUG  ! summarize new headers
*        do i = 1, n_ihead
*          write(nu_dbg,'(a,i2,a,i2,a,i4)') ' for i = ',i,
*     :      ' int_head(', indx_ihead(i),') = ', new_i_head(i)
*        end do
*        do i = 1, n_rhead
*          write(nu_dbg,'(a,i2,a,i2,a,1pe10.3)') ' for i = ',i,
*     :      ' real_head(', indx_rhead(i),') = ', anew_r_head(i)
*        end do
* DEBUG

* Get headers, determine if the file is a time series or response spectra file:
        call ReadHdrs(f_smc(1:nc_f_smc), char_head, int_head, 
     :                real_head, comments)
* DEBUG
*        write(nu_dbg,'(a)') '  Beyond Readhdrs'
*        write(nu_dbg,'(a)') ' For file '//f_smc(1:nc_f_smc)
*        write(nu_dbg,'(a,1x,i5)') '  ncomments = ', int_head(16)
*        write(nu_dbg,'(a)') '  char_head(1) = '
*        write(nu_dbg,'(a)')    char_head(1)
*        write(nu_dbg,'(a)') '  char_head(1)(1:1) = '//char_head(1)(1:1)
* DEBUG

        if (char_head(1)(1:1) == '5') then  ! response spectra
* DEBUG
*          write(nu_dbg,'(a)') '  Before rs2read'
* DEBUG

          call rs2read(f_smc(1:nc_f_smc), 
     :        damp, ndamp, per, nper, rd, rv, psv, aa, 
     :        char_head, int_head, real_head, comments,
     :        itflag, tprd, tprv, tpaa)
        else
          call SMC_Npts(f_smc(1:nc_f_smc),npts,sps)
          allocate( y(npts))
          tskip = 0.0
          tlength = -1.0
          call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :             y, char_head, int_head, real_head, comments)
      precision = ' '
      if (int_head(47) == 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
        end if

        iflag = 0 ! not used by current PC version of datetime; Chris
                  ! Stephens would modify datetime.for to include Unix routines
        call datetime(date_c, time_c, iflag)
        ncomments = int_head(16)
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        comments(ncomments) = '| Using program SMCNuHDR:'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '|   The following headers replaced on '//date_c//
     :  ' at time '//time_c//':'

        do j = 1, n_ihead
          int_head(indx_ihead(j)) = new_i_head(j)
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i2,a)') 
     :     '|   int_head(', indx_ihead(j), ')'
        end do            

        do j = 1, n_rhead
          real_head(indx_rhead(j)) = anew_r_head(j)
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i2,a)') 
     :     '|   real_head(', indx_rhead(j), ')'
        end do            

        do j = 1, n_thead
          char_head(indx_thead(j)) = thead_new(j)(1:nc_thead_new(j))
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,i2,a)') 
     :     '|   text_head(', indx_thead(j), ')'
        end do            

        ncomments = ncomments + 1
        comments(ncomments) = '|'
        int_head(16) = ncomments
        itype = -1
        if (char_head(1)(1:1) == '5') then  ! response spectra
          call rs2write(f_smc(1:nc_f_smc), 
     :        damp, ndamp, per, nper, rd, rv, aa, 
     :        char_head, int_head, real_head, comments, 
     :        itflag, tprd, tprv, tpaa)
        else
          call SMCWrite(f_smc(1:nc_f_smc), precision,
     :                y, char_head, itype, 
     :                 int_head, real_head, comments)
          deallocate( y)        
        end if

* Finished with file, go back for another file or another set of headers
        go to 3000

      end if
     
9999  continue
      close(nu_ctl)
      stop
      end
* ---------------------------------------------------------------- SMCNuHdr


      include '\forprogs\rc_subs.for'
      include '\forprogs\skip.for'
      include '\forprogs\doy.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\rs2read.for'
      include '\forprogs\rs2write.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmax.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\distaz.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\datetime.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\readhdrs.for'


                     